@@ -52,12 +52,6 @@ public class RawBatchConverter {
     }
 
     public static List<ImmutableTriple<MessageId, String, Integer>> extractIdsAndKeysAndSize(RawMessage msg)
-        throws IOException {
-        return extractIdsAndKeysAndSize(msg, true);
-    }
-
-    public static List<ImmutableTriple<MessageId, String, Integer>> extractIdsAndKeysAndSize(RawMessage msg,
-                                                                                             boolean extractNullKey)
             throws IOException {
         checkArgument(msg.getMessageIdData().getBatchIndex() == -1);
 
@@ -81,10 +75,10 @@ public class RawBatchConverter {
                                                   msg.getMessageIdData().getEntryId(),
                                                   msg.getMessageIdData().getPartition(),
                                                   i);
-            if (!smm.isCompactedOut() && (extractNullKey || smm.hasPartitionKey())) {
+            if (!smm.isCompactedOut()) {
                 idsAndKeysAndSize.add(ImmutableTriple.of(id,
-                        smm.hasPartitionKey() ? smm.getPartitionKey() : null,
-                        smm.hasPayloadSize() ? smm.getPayloadSize() : 0));
+                            smm.hasPartitionKey() ? smm.getPartitionKey() : null,
+                            smm.hasPayloadSize() ? smm.getPayloadSize() : 0));
             }
             singleMessagePayload.release();
         }
