@@ -18,18 +18,10 @@
  */
 package org.apache.pulsar.client.impl.auth;
 
-import static org.mockito.ArgumentMatchers.any;
-import static org.mockito.ArgumentMatchers.anyBoolean;
-import static org.mockito.ArgumentMatchers.anyInt;
-import static org.mockito.Mockito.mock;
-import static org.mockito.Mockito.when;
 import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertFalse;
-import static org.testng.Assert.assertNull;
 import static org.testng.Assert.assertTrue;
 import static org.testng.Assert.fail;
-import org.mockito.MockedConstruction;
-import org.mockito.Mockito;
 import org.testng.annotations.Test;
 import org.apache.pulsar.common.util.ObjectMapperFactory;
 import static org.apache.pulsar.common.util.Codec.encode;
@@ -295,53 +287,4 @@ public class AuthenticationAthenzTest {
         assertEquals(auth2.getAuthData().getHttpHeaders().iterator().next().getKey(), "Test-Role-Header");
         auth2.close();
     }
-
-    @Test
-    public void testZtsProxyUrlSetting() throws Exception {
-        final String ztsProxyUrl = "https://example.com:4443/";
-        final String paramsStr = new String(Files.readAllBytes(Paths.get("./src/test/resources/authParams.json")));
-        final ObjectMapper jsonMapper = ObjectMapperFactory.create();
-        final Map<String, String> authParamsMap = jsonMapper.readValue(paramsStr, new TypeReference<HashMap<String, String>>() { });
-
-        try (MockedConstruction<ZTSClient> mockedZTSClient = Mockito.mockConstruction(ZTSClient.class, (mock, context) -> {
-            final String actualZtsProxyUrl = (String) context.arguments().get(1);
-            assertNull(actualZtsProxyUrl);
-
-            when(mock.getRoleToken(any(), any(), anyInt(), anyInt(), anyBoolean())).thenReturn(mock(RoleToken.class));
-        })) {
-            authParamsMap.remove("ztsProxyUrl");
-            final AuthenticationAthenz auth1 = new AuthenticationAthenz();
-            auth1.configure(jsonMapper.writeValueAsString(authParamsMap));
-            auth1.getAuthData();
-
-            assertEquals(mockedZTSClient.constructed().size(), 1);
-
-            auth1.close();
-
-            authParamsMap.put("ztsProxyUrl", "");
-            final AuthenticationAthenz auth2 = new AuthenticationAthenz();
-            auth2.configure(jsonMapper.writeValueAsString(authParamsMap));
-            auth2.getAuthData();
-
-            assertEquals(mockedZTSClient.constructed().size(), 2);
-
-            auth2.close();
-        }
-
-        try (MockedConstruction<ZTSClient> mockedZTSClient = Mockito.mockConstruction(ZTSClient.class, (mock, context) -> {
-            final String actualZtsProxyUrl = (String) context.arguments().get(1);
-            assertEquals(actualZtsProxyUrl, ztsProxyUrl);
-
-            when(mock.getRoleToken(any(), any(), anyInt(), anyInt(), anyBoolean())).thenReturn(mock(RoleToken.class));
-        })) {
-            authParamsMap.put("ztsProxyUrl", ztsProxyUrl);
-            final AuthenticationAthenz auth3 = new AuthenticationAthenz();
-            auth3.configure(jsonMapper.writeValueAsString(authParamsMap));
-            auth3.getAuthData();
-
-            assertEquals(mockedZTSClient.constructed().size(), 1);
-
-            auth3.close();
-        }
-    }
 }
