@@ -357,7 +357,7 @@ public class PulsarClientToolTest extends BrokerTestBase {
                 "--memory-limit", memoryLimitArg,
                 "produce", "-m", message,
                 "-n", Integer.toString(numberOfMessages), topicName};
-        pulsarClientTool.jcommander.parse(args);
+        pulsarClientTool.getCommander().parseArgs(args);
         assertEquals(pulsarClientTool.rootParams.getTlsTrustCertsFilePath(), CA_CERT_FILE_PATH);
         assertEquals(pulsarClientTool.rootParams.getAuthParams(), authParams);
         assertEquals(pulsarClientTool.rootParams.getAuthPluginClassName(), authPlugin);
@@ -386,8 +386,7 @@ public class PulsarClientToolTest extends BrokerTestBase {
                 "-ml", memoryLimitArg,
                 "produce", "-m", message,
                 "-n", Integer.toString(numberOfMessages), topicName};
-        
-        pulsarClientTool.jcommander.parse(args);
+        pulsarClientTool.getCommander().parseArgs(args);
         assertEquals(pulsarClientTool.rootParams.getMemoryLimit(), 10 * 1024 * 1024);
     }
 
@@ -405,7 +404,7 @@ public class PulsarClientToolTest extends BrokerTestBase {
         String[] args = {"--url", url,
                 "produce", "-m", message,
                 "-n", Integer.toString(numberOfMessages), topicName};
-        pulsarClientTool.jcommander.parse(args);
+        pulsarClientTool.getCommander().parseArgs(args);
         assertEquals(pulsarClientTool.rootParams.getServiceURL(), url);
         assertEquals(pulsarClientTool.rootParams.getProxyServiceURL(), "pulsar+ssl://my-proxy-pulsar:4443");
         assertEquals(pulsarClientTool.rootParams.getProxyProtocol(), ProxyProtocol.SNI);
