@@ -20,6 +20,7 @@ package org.apache.pulsar.client.impl.conf;
 
 import static com.google.common.base.Preconditions.checkArgument;
 import com.fasterxml.jackson.annotation.JsonIgnore;
+import com.fasterxml.jackson.annotation.JsonProperty;
 import com.google.common.collect.Sets;
 import io.swagger.annotations.ApiModelProperty;
 import java.io.Serializable;
@@ -65,7 +66,7 @@ public class ConsumerConfigurationData<T> implements Serializable, Cloneable {
 
     @ApiModelProperty(
             name = "topicsPattern",
-            value = "Topic pattern"
+            value = "The regexp for the topic name(not contains partition suffix)."
     )
     private Pattern topicsPattern;
 
@@ -355,7 +356,7 @@ public class ConsumerConfigurationData<T> implements Serializable, Cloneable {
                     + "When specifying the dead letter policy while not specifying `ackTimeoutMillis`, you can set the"
                     + " ack timeout to 30000 millisecond."
     )
-    private transient DeadLetterPolicy deadLetterPolicy;
+    private DeadLetterPolicy deadLetterPolicy;
 
     private boolean retryEnable = false;
 
@@ -378,12 +379,13 @@ public class ConsumerConfigurationData<T> implements Serializable, Cloneable {
             value = "If `replicateSubscriptionState` is enabled, a subscription state is replicated to geo-replicated"
                     + " clusters."
     )
-    private boolean replicateSubscriptionState = false;
+    @JsonProperty(access = JsonProperty.Access.READ_WRITE)
+    private Boolean replicateSubscriptionState;
 
     private boolean resetIncludeHead = false;
 
     @JsonIgnore
-    private transient KeySharedPolicy keySharedPolicy;
+    private KeySharedPolicy keySharedPolicy;
 
     private boolean batchIndexAckEnabled = false;
 
@@ -434,4 +436,14 @@ public class ConsumerConfigurationData<T> implements Serializable, Cloneable {
             throw new RuntimeException("Failed to clone ConsumerConfigurationData");
         }
     }
+
+    /**
+     * Backward compatibility with the old `replicateSubscriptionState` field.
+     * @deprecated Using {@link #getReplicateSubscriptionState()} instead.
+     */
+    @JsonIgnore
+    @Deprecated
+    public boolean isReplicateSubscriptionState() {
+        return replicateSubscriptionState != null && replicateSubscriptionState;
+    }
 }
